/*
 *  $Id: make_graph.c,v 1.5 1993/11/12 12:35:46 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Make_graph - make a new communicator to which topology information
                 has been attached

algorithm:
We ignore the periods and reorder info currently.

@*/
int MPI_Make_graph ( comm_old, nnodes, index, edges, reorder, comm_graph )
MPI_Comm  comm_old;
int               nnodes;
int              *index;
int              *edges;
int               reorder;
MPI_Comm *comm_graph;
{
  int range[3];
  MPI_Group group_old, group;
  int i, rank, num_ranks = 1;

  /* Check validity of arguments */
  if ( (comm_old   == (MPI_Comm)0 ) ||
       (comm_graph == (MPI_Comm *)0) ||
       (index      == (int *)0) ||
       (edges      == (int *)0) ||
       (nnodes     <  1) )
    return (MPI_SUCCESS);
  
  /* Determine number of ranks in topology and make new comm */
  num_ranks = nnodes;
  if ( num_ranks < 1 ) {
    (*comm_graph)  = (MPI_Comm)0;
    return (MPI_SUCCESS);
  }
  range[0] = 0; range[1] = num_ranks - 1; range[2] = 1;
  MPI_Comm_group ( comm_old, &group_old );
  MPI_Group_range_incl ( group_old, 1, range, &group );
  MPI_Comm_make  ( comm_old, group, comm_graph );

  /* Store topology information in new communicator */
  if ( (*comm_graph) != (MPI_Comm)0 ) {
    (*comm_graph)->topology.type                 = MPI_GRAPH;
    (*comm_graph)->topology.tag.graph.nnodes     = nnodes;
    (*comm_graph)->topology.tag.graph.nedges     = index[nnodes-1];
    (*comm_graph)->topology.tag.graph.index      = 
      (int *)MPIR_MALLOC( sizeof(int) * (nnodes + index[nnodes-1]) );
    (*comm_graph)->topology.tag.graph.edges      = 
      (*comm_graph)->topology.tag.graph.index + nnodes;
    for ( i=0; i<nnodes; i++ )
      (*comm_graph)->topology.tag.graph.index[i] = index[i];
    for ( i=0; i<index[nnodes-1]; i++ )
      (*comm_graph)->topology.tag.graph.edges[i] = edges[i];
  }
  return (MPI_SUCCESS);
}
